<?php

require_once WICKED_BASE . '/lib/Page/StandardPage.php';

/**
 * Wicked AttachedFiles class.
 *
 * $Horde: wicked/lib/Page/AttachedFiles.php,v 1.4 2004/12/01 16:28:08 eraserhd Exp $
 *
 * @author  Jason M. Felice <jfelice@cronosys.com>
 * @version $Revision: 1.4 $
 * @package Wicked
 */
class AttachedFiles extends Page {

    /**
     * Display modes supported by this page.
     * @var array $supportedModes
     */
    var $supportedModes = array(
        WICKED_MODE_CONTENT => true,
        WICKED_MODE_DISPLAY => true);

    /**
     * The page for which we'd like to manipulate attachments.
     * @var string $_referrer
     */
    var $_referrer = null;

    function AttachedFiles($referrer)
    {
        $this->_referrer = $referrer;
    }

    /** 
     * Retrieve this user's permissions for the referring page.
     *
     * @return integer  The permissions bitmask.
     */
    function getPermissions()
    {
        $page = &Page::getPage($this->referrer(), null, null, null, null);
        return $page->getPermissions();
    }

    /**
     * Render this page in Content mode.
     *
     * @return string  The page content, or PEAR_Error.
     */
    function content()
    {
        global $wicked;

        $attachments = $wicked->getAttachedFiles($this->referrer(), true);
        if (is_a($attachments, 'PEAR_Error')) {
            return $attachments;
        }

        foreach ($attachments as $idx => $attach) {
            $attachments[$idx]['date'] = date('M j, Y g:ia',
                                              $attach['attachment_created']);
            $url = Horde::applicationUrl('attachment.php');
            $url = Util::addParameter($url, array('page' => $this->referrer()));
            $url = Util::addParameter($url, array('file' => $attach['attachment_name']));
            $url = Util::addParameter($url, array('version' => $attach['attachment_majorversion'] . '.' . $attach['attachment_minorversion']));
            $attachments[$idx]['url'] = $url;
        }

        return $attachments;
    }

    /**
     * Render this page in Display mode.
     *
     * @return mixed  Returns true or PEAR_Error.
     */
    function display()
    {
        global $registry;

        $attachments = $this->content();
        if (is_a($attachments, 'PEAR_Error')) {
            global $notification;
            $notification->push('Error retrieving attachments: ' .
                                $attachments->getMessage(), 'horde.error');
            return $attachments;
        }

        require_once 'Horde/Template.php';
        $template = &new Horde_Template();

        $template->setOption('gettext', true);
        $template->set('pageName', $this->pageName());

        $refreshIcon = Horde::link($this->pageUrl('display.php')) .
                       Horde::img('reload.png',
                                  sprintf(_("Reload %s"), $this->pageName()),
                                  '', $registry->getImageDir('horde')).
                       '</a>';
        $template->set('refreshIcon', $refreshIcon);
        $template->set('attachments', $attachments);

        /* Get an array of unique filenames for the update form. */
        $files = array();
        foreach ($attachments as $attachment) {
            $files[$attachment['attachment_name']] = true;
        }
        $files = array_keys($files);
        sort($files);
        $template->set('files', $files);
        $template->set('canUpdate', count($files) > 0, true);

        $requiredMarker = Horde::img('required.png', '*', '',
                                     $registry->getImageDir('horde'));
        $template->set('requiredMarker', $requiredMarker);
        $template->set('referrer', $this->referrer());
        $template->set('formInput', Util::formInput());

        echo $template->fetch(WICKED_TEMPLATES . '/display/AttachedFiles.html');
    }

    function pageName()
    {
        return sprintf(_("AttachedFiles: %s"), $this->referrer());
    }

    function referrer()
    {
        return $this->_referrer;
    }

    function pageUrl($linkpage = 'display.php', $actionid = 'none')
    {
        $url = Horde::applicationUrl($linkpage);
        $url = Util::addParameter($url, 'page', 'AttachedFiles');
        $url = Util::addParameter($url, 'referrer', $this->referrer());
        return Util::addParameter($url, 'actionID', $actionid);
    }

    /**
     * Retrieve the form fields and process the attachment.
     */
    function handleAction()
    {
        global $notification, $wicked;

        $is_update = Util::getFormData('is_update');
        $filename = Util::getFormData('filename');
        $change_log = Util::getFormData('change_log');

        if (empty($filename)) {
            $filename = $_FILES['attachment_file']['name'];
        }

        $fh = @fopen($_FILES['attachment_file']['tmp_name'], "r");
        if (!is_resource($fh)) {
            $notification->push(_("Can't read uploaded file."), 'horde.error');
            return;
        }
        $data = fread($fh, filesize($_FILES['attachment_file']['tmp_name']));
        fclose($fh);

        $attachments = $wicked->getAttachedFiles($this->referrer());
        $found = false;
        foreach ($attachments as $attach) {
            if ($filename == $attach['attachment_name']) {
                $found = true;
                break;
            }
        }

        $minor_change = false;
        if ($is_update == 'true') {
            if (!$found) {
                $notification->push(sprintf(_("Can't update '%s': no such attachment."), $filename), 'horde.error');
                return;
            }
            $minor_change = Util::getFormData('minor_change');
        } else {
            if ($found) {
                $notification->push(sprintf(_("There is already an attachment named '%s'."), $filename), 'horde.error');
                return;
            }
        }

        $file = array('page_name'       => $this->referrer(),
                      'attachment_name' => $filename,
                      'minor'           => $minor_change,
                      'change_log'      => $change_log);

        $res = $wicked->attachFile($file, $data);
        if (is_a($res, 'PEAR_Error')) {
            $notification->push($res, 'horde.error');
            return $res;
        }
    }

}
