<?php
/**
 * Wicked Page class for most pages.
 *
 * $Horde: wicked/lib/Page/StandardPage.php,v 1.48 2004/12/01 16:18:11 eraserhd Exp $
 *
 * @author  Tyler Colbert <tyler-hordeml@colberts.us>
 * @version $Revision: 1.48 $
 * @package Wicked
 */
class StandardPage extends Page {

    /**
     * Display modes supported by this page.
     * @var array $supportedModes
     */
    var $supportedModes = array(
        WICKED_MODE_DISPLAY => true,
        WICKED_MODE_EDIT => true,
        WICKED_MODE_REMOVE => true,
        WICKED_MODE_HISTORY => true,
        WICKED_MODE_DIFF => true,
        WICKED_MODE_LOCKING => true);

    /**
     * Construct a standard page class to represent a wiki page.
     *
     * @param string  $pagename    The name of the page to represent.
     * @param boolean $autocreate  Create the page automatically if it doesn't exist
     *                             and we have edit permissions? Defaults to false.
     */
    function StandardPage($pagename, $autocreate = false, $template = null)
    {
        if (is_array($pagename)) {
            $this->_page = $pagename;
            return;
        }

        global $wicked, $notification, $perms;
        $page = $wicked->retrieveByName($pagename, ($autocreate && $this->allows(WICKED_MODE_EDIT)), $template);

        // make sure 'wicked' permission exists.  Set reasonable defaults
        // if necessary.
        if (!$perms->exists('wicked')) {
            $perm = &$perms->newPermission('wicked');
            $perm->addGuestPermission(PERMS_SHOW | PERMS_READ);
            $perm->addDefaultPermission(PERMS_SHOW | PERMS_READ | PERMS_EDIT | PERMS_DELETE);
        }

        // make sure 'wicked:pages' exists.  Copy from 'wicked' if it does
        // not exist.
        if (!$perms->exists('wicked:pages')) {
            $perm = &$perms->newPermission('wicked:pages');
            $copyFrom = &$perms->getPermission('wicked');
            $perm->addGuestPermission($copyFrom->getGuestPermissions());
            $perm->addDefaultPermission($copyFrom->getDefaultPermissions());
            $perm->addCreatorPermission($copyFrom->getCreatorPermissions());
            foreach ($copyFrom->getUserPermissions() as $user => $uperm) {
                $perm->addUserPermission($user, $uperm, false);
            }
            foreach ($copyFrom->getGroupPermissions() as $group => $gperm) {
                $perm->addGroupPermission($group, $gperm, false);
            }
        } 

        /* If we can't load $pagename, or didn't autocreate it, load
         * the AddPage page to give the user the option to create it,
         * unless $pagename is 'WikiHome'. */
        if (is_a($page, 'PEAR_Error') && $pagename == 'WikiHome') {
            /* Always try to autocreate WikiHome and fail if we
             * can't. */
            $page = $wicked->retrieveByName('WikiHome', true);
            if (is_a($page, 'PEAR_Error')) {
                Horde::fatal($page, __FILE__, __LINE__);
            }
        }

        if (is_a($page, 'PEAR_Error')) {
            $this->_page = array();
        } else {
            $this->_page = $page;
        }
    }

    function displayContents($isBlock)
    {
        $wiki = &$this->getProcessor();
        $text = $wiki->transform($this->getText());

        ob_start();
        require WICKED_TEMPLATES . '/display/standard.inc';
        $result = ob_get_contents();
        ob_end_clean();
        return $result;
    }

    /**
     * Render this page in History mode.
     *
     * @return mixed  Returns true or PEAR_Error.
     */
    function history()
    {
        require_once WICKED_BASE . '/lib/Page/StandardPage/StdHistoryPage.php';

        global $wicked;
        $summaries = $wicked->getHistory($this->pageName());
        if (is_a($summaries, 'PEAR_Error')) {
            $GLOBALS['notification']->push('Error retrieving histories : ' . $summaries->toString(), 'horde.error');
            return $summaries;
        }

        // Header.
        $show_restore = !$this->isLocked();
        $allow_diff = true;
        $show_edit = true;
        require WICKED_TEMPLATES . '/history/header.inc';
        $style = 'text';

        // First item is this page.
        $show_restore = false;
        $page = $this;
        $i = 0;
        require WICKED_TEMPLATES . '/history/summary.inc';

        // Now the rest of the histories.
        $show_restore = !$this->isLocked();
        $show_edit = false;
        foreach ($summaries as $page) {
            $i++;
            $page = &new StdHistoryPage($page);
            require WICKED_TEMPLATES . '/history/summary.inc';
        }

        // Footer.
        require WICKED_TEMPLATES . '/history/footer.inc';
    }

    function isLocked()
    {
        return !empty($this->_page['page_locked']);
    }

    function lock()
    {
        global $wicked;
        if ($wicked->lock($this->pageName(), true)) {
            $this->_page['page_locked'] = true;
        }
    }

    function unlock()
    {
        global $wicked;
        if ($wicked->lock($this->pageName(), false)) {
            $this->_page['page_locked'] = false;
        }
    }

    function updateText($newtext, $changelog, $minorchange)
    {
        global $wicked, $conf, $registry;

        $version = $this->version();
        $result = $wicked->updateText($this->pageName(), $newtext, $changelog, $minorchange);
        if (!is_a($result, 'PEAR_Error')) {
            $result = true;
            if (!empty($conf['wicked']['notify_address'])) {
                require_once 'Horde/MIME.php';

                $new_page = &$this->getPage($this->pageName());
                $url = Horde::applicationUrl('display.php', true, -1);
                $url = Util::addParameter($url, 'page', $this->pageName());
                $diff = $new_page->getDiff($version);

                if (Auth::getAuth()) {
                    $message = Auth::getAuth();
                } else {
                    $message = 'guest [' . $_SERVER['REMOTE_ADDR'] . ']';
                }

                $lc_time = setlocale(LC_TIME, 'C');
                $message .= '  ' . date('r') . "\n\n";
                setlocale(LC_TIME, $lc_time);

                $message .= "Modified page: $url\n";
                $message .= 'New Revision:  ' . $new_page->version() . "\n\n";
                $message .= $diff;

                $headers = array('Subject' => MIME::encode('[' . $registry->get('name') . '] changed: ' . $this->pageName(), NLS::getCharset()));

                $result = Wicked::mail($message, $headers);
            }
            $this->_page['page_text'] = $newtext;
        }

        return $result;
    }

    function pageName()
    {
        return isset($this->_page['page_name']) ? $this->_page['page_name'] : '';
    }

    function getText()
    {
        return isset($this->_page['page_text']) ? $this->_page['page_text'] : '';
    }

    function versionCreated()
    {
        return isset($this->_page['version_created']) ? $this->_page['version_created'] : '';
    }

    function author()
    {
        $modify = $this->_page['change_author'];
        if (empty($modify)) {
            return _("Guest");
        }

        require_once 'Horde/Identity.php';
        $identity = &Identity::singleton(null, $modify);
        $name = $identity->getValue('fullname');
        if (!empty($name)) {
            $modify = $name;
        }

        return $modify;
    }

    function hits()
    {
        return !empty($this->_page['page_hits']) ? $this->_page['page_hits'] : 0;
    }

    function changeLog()
    {
        return $this->_page['change_log'];
    }

    function version()
    {
        return $this->_page['page_majorversion'] . '.' . $this->_page['page_minorversion'];
    }

    function diff($version)
    {
        require WICKED_TEMPLATES . '/diff/diff.inc';
    }

    /**
     * Produce a diff for this page.
     *
     * @param string $version           Previous version, or null if diffing
     *                                  with `before the beginning' (empty).
     * @param optional string $renderer The diff renderer.
     */
    function getDiff($version, $renderer = 'unified')
    {
        if (is_null($version)) {
            $old_page_text = '';
        } else {
            $old_page = &$this->getPage($this->pageName(), $version);
            $old_page_text = $old_page->getText();
        }

        include_once 'Text/Diff.php';
        include_once 'Text/Diff/Renderer.php';
        include_once 'Text/Diff/Renderer/' . $renderer . '.php';

        $diff = &new Text_Diff(explode("\n", $old_page_text),
                               explode("\n", $this->getText()));

        /* Return the diff just in unified format for now. */
        $class = 'Text_Diff_Renderer_' . $renderer;
        $renderer = &new $class();
        return $renderer->render($diff);
    }

    function removeVersion($version)
    {
        return $GLOBALS['wicked']->removeVersion($this->pageName(), $version);
    }

    function removeAllVersions()
    {
        return $GLOBALS['wicked']->removeAllVersions($this->pageName());
    }

}
